//=============================================================================
// EmitterActor.
//=============================================================================
class EmitterActor expands ConvasActor;

struct SmartColor
{
	var float R,B,G;
};
var struct Particle
{
	var vector PartLoc,PartVel,PartAccel;
	var SmartColor PartDestColor,PartBegColor;
	var color PartColor;
	var byte PartStyle;
	var float PartMaxScale,FadeInTime,FadeOutTime,FullOnTime,PartScale,BegFI,BegFO,MaxLife,LifeTime;
	var texture ParticleTex;
} Particles[200];
var(EmSpawning) float EmittingDelay,EmittingRand;
var(EmSpawning) vector EmitterVelocity,EmitterVelRand,EmitterAccel,EmitterRandOffset,EmitterOffset;
var(EmGeneral) bool bEnabled,bParticlesFollowMe,bCoronaEmitter;
var(EmGeneral) float CoronaMaxScale,EmitterSize;
var(EmTexture) texture ParticleTexture;
var(EmTexture) color ParticleColor,EndPartColor;
var(EmTexture) ERenderStyle EmitterStyle;
var(EmFading) float FadesInTime,StayOnTime,FadeOutTime,StayOnRand;
var byte NumParticles;
var bool bLastSucceeded,bHasTimed;
var float FadeInTime;

replication
{
	unreliable if( Role==ROLE_Authority )
		bEnabled;
}

simulated function PostBeginPlay()
{
	DrawType = DT_None;
	SetTimer(0.1,False);
}
simulated function RenderCanvas( Canvas C, bool bSeenBefore, vector CamV, rotator CamR )
{
	local float X,Y;
	local int i;
	local vector ParL;
	
	if( !bSeenBefore )
	{
		FadeInTime = 0;
		bLastSucceeded = True;
		Return;
	}
	if( NumParticles==0 )
	{
		if( WorldToScreen(Location,C.ViewPort.Actor,CamV,CamR,C.ClipX,C.ClipY,X,Y) &&
		 X>0 && Y>0 && X<C.ClipX && Y<C.ClipY )
		{
			FadeInTime = 0.01;
			bLastSucceeded = True;
			Return;
		}
	}
	For( i=0; i<NumParticles; i++ )
	{
		if( bParticlesFollowMe )
			ParL = Particles[i].PartLoc+Location;
		else ParL = Particles[i].PartLoc;
		if( WorldToScreen(ParL,C.ViewPort.Actor,CamV,CamR,C.ClipX,C.ClipY,X,Y) &&
		 X>0 && Y>0 && X<C.ClipX && Y<C.ClipY )
		{
			DrawParticle(C,Particles[i],X,Y,VSize(CamV-ParL));
			bLastSucceeded = True;
		}
	}
	C.Reset();
}
simulated function DrawParticle( Canvas C, Particle Par, float X, float Y, float Dist )
{
	local float S;
	
	if( Dist>5000 )
		Return;
	S = (5000-Dist)/(5000/Par.PartScale);
	if( bCoronaEmitter && S>Par.PartMaxScale )
		S = Par.PartMaxScale;
	C.DrawColor = Par.PartColor;
	C.Style = Par.PartStyle;
	C.SetPos(X-(Par.ParticleTex.VSize*S/2),Y-(Par.ParticleTex.USize*S/2));
	C.DrawIcon(Par.ParticleTex,S);
}
simulated function Tick( float D )
{
	local int i,ii;
	local float CF,Ch;
	local SmartColor NewColor;
	
	if( !bLastSucceeded )
	{
		FadeInTime = 0;
		Return;
	}
	bLastSucceeded = False;
	if( NumParticles==0 )
	{
		FadeInTime = 0.01;
		Return;
	}
	if( FadeInTime<1 )
	{
		FadeInTime+=D*3;
		if( FadeInTime>1 )
			FadeInTime = 1;
	}
	For( i=0; i<NumParticles; i++ )
	{
		if( Particles[i].LifeTime>=Particles[i].MaxLife ) // Emitter died, kill me.
		{
			if( (NumParticles+1)==i )
			{
				NumParticles--;
				Return;
			}
			else
			{
				For( ii=i; ii<(NumParticles-1); ii++ )
					Particles[ii] = Particles[ii+1];
				NumParticles--;
			}
			Continue;
		}
		if( Particles[i].FadeInTime>0 )
		{
			Particles[i].FadeInTime-=D;
			CF = 1-(Particles[i].FadeInTime/Particles[i].BegFI);
		}
		else if( Particles[i].FullOnTime>0 )
		{
			Particles[i].FullOnTime-=D;
			CF = 1;
		}
		else if( Particles[i].FadeOutTime>0 )
		{
			Particles[i].FadeOutTime-=D;
			CF = Particles[i].FadeOutTime/Particles[i].BegFO;
		}
		CF*=FadeInTime;
		if( CF<0 )
			CF = 0;
		else if( CF>1 )
			CF = 1;
		Particles[i].LifeTime+=D;
		Ch = Particles[i].LifeTime/Particles[i].MaxLife;
		NewColor = AddTo(MultiplyWith(Particles[i].PartBegColor,(1-Ch)),MultiplyWith(Particles[i].PartDestColor,Ch));
		NewColor = MultiplyWith(NewColor,CF);
		Particles[i].PartColor = ConvertTo(NewColor);
		Particles[i].PartVel+=Particles[i].PartAccel*D;
		Particles[i].PartLoc+=Particles[i].PartVel*D;
	}
}
simulated function SmartColor MultiplyWith( SmartColor A, float B )
{
	A.R*=B;
	A.B*=B;
	A.G*=B;
	Return A;
}
simulated function SmartColor AddTo( SmartColor A, SmartColor B )
{
	A.R+=B.R;
	A.B+=B.B;
	A.G+=B.G;
	Return A;
}
simulated function color ConvertTo( SmartColor B )
{
	local color A;
	
	A.R = B.R;
	A.B = B.B;
	A.G = B.G;
	Return A;
}
simulated function SmartColor ConvertFrom( color B )
{
	local SmartColor A;
	
	A.R = B.R;
	A.B = B.B;
	A.G = B.G;
	Return A;
}
simulated function Timer()
{
	if( !bHasTimed )
	{
		bHasTimed = True;
		if( Class'StaticInfo'.Static.IsDedicatedServer(Self) )
		{
			Disable('Tick');
			Return; // Don't handel the emitter on dedicated server... useless...
		}
	}
	if( !bEnabled )
	{
		SetTimer(1,False);
		Return;
	}
	if( EmittingRand!=0 )
		SetTimer(EmittingDelay+FRand()*EmittingRand,False);
	else SetTimer(EmittingDelay,False);
	if( FadeInTime==0 ) Return;
	if( NumParticles==200 ) Return; // Max particles is 200.
	if( bParticlesFollowMe )
		Particles[NumParticles].PartLoc = EmitterOffset;
	else Particles[NumParticles].PartLoc = Location+EmitterOffset;
	Particles[NumParticles].PartLoc = AddRandomize(Particles[NumParticles].PartLoc,EmitterRandOffset);
	Particles[NumParticles].PartVel = AddRandomize(EmitterVelocity,EmitterVelRand);
	Particles[NumParticles].PartAccel = EmitterAccel;
	Particles[NumParticles].PartBegColor = ConvertFrom(ParticleColor);
	Particles[NumParticles].PartDestColor = ConvertFrom(EndPartColor);
	Particles[NumParticles].PartColor.R = 0;
	Particles[NumParticles].PartColor.B = 0;
	Particles[NumParticles].PartColor.G = 0;
	Particles[NumParticles].PartMaxScale = CoronaMaxScale;
	Particles[NumParticles].PartScale = EmitterSize;
	Particles[NumParticles].FadeInTime = FadesInTime;
	Particles[NumParticles].BegFI = FadesInTime;
	Particles[NumParticles].FadeOutTime = FadeOutTime;
	Particles[NumParticles].BegFO = FadeOutTime;
	Particles[NumParticles].FullOnTime = StayOnTime+StayOnRand*FRand();
	Particles[NumParticles].PartStyle = (EmitterStyle);
	Particles[NumParticles].ParticleTex = ParticleTexture;
	Particles[NumParticles].MaxLife = Particles[NumParticles].BegFI+Particles[NumParticles].BegFO+Particles[NumParticles].FullOnTime;
	Particles[NumParticles].LifeTime = 0;
	NumParticles++;
}
simulated function vector AddRandomize( vector Def, vector Ran )
{
	if( Ran.X!=0 )
		Def.X+=(Ran.X*2*FRand())-Ran.X;
	if( Ran.Y!=0 )
		Def.Y+=(Ran.Y*2*FRand())-Ran.Y;
	if( Ran.Z!=0 )
		Def.Z+=(Ran.Z*2*FRand())-Ran.Z;
	Return Def;
}

defaultproperties
{
				bHidden=False
				bNoDelete=True
				bMovable=False
				Texture=Texture'UnrealShare.Icons.GreenSkull'
				bAlwaysRelevant=True
}
